unit fMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, uList, StdCtrls, FileCtrl;


type
  TDirTreeNode  = class (TDListItem)
  protected
    m_Space: integer;
    m_SpaceByFilesOnly: integer;
    m_Children: TDList;
    m_Name: string;

  public
    { instantiation }
    constructor Create;
    destructor Destroy; override;

    { accessors }
    property Space: integer read m_Space;

    { operations }

    procedure Load (Path: string);
    procedure Clear;
    procedure ToTreeView  (tnl: TTreeNodes; tn: TTreeNode; redAlert: boolean);

  private
  end;


  TfmMain = class(TForm)
    bnRefresh: TButton;
    tvDir: TTreeView;
    dcbDrives: TDriveComboBox;
    flbFiles: TFileListBox;
    pbScan: TProgressBar;
    ImageList1: TImageList;
    Label1: TLabel;
    Label2: TLabel;
    Button1: TButton;
    Label3: TLabel;
    Button2: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure bnRefreshClick(Sender: TObject);
    procedure tvDirChange(Sender: TObject; Node: TTreeNode);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private declarations }
    dirTree : TDirTreeNode;
  public
    { Public declarations }
  end;

var
  fmMain: TfmMain;

  iCurrentSize: integer;     // to update the progress bar

implementation

uses fAboutDS, fHelp;

{$R *.DFM}

{-----------------------------------------------------------------------------}
{--------------------  TDirTreeNode  -----------------------------------------}
{-----------------------------------------------------------------------------}

constructor TDirTreeNode.Create;
begin
  inherited Create;
  m_Space := 0;
  m_SpaceByFilesOnly := 0;
  m_Children := TDList.Create (true);
  m_Name := '\';
end;

destructor TDirTreeNode.Destroy;
begin
  m_Children.Free;
end;


{ operations }

procedure TDirTreeNode.Load (Path: string);
var
  Found: integer;
  SearchRec: TSearchRec;
  dtn: TDirTreeNode;
begin
  Clear;
  Found := FindFirst(Path + '*.*' , faAnyFile, SearchRec);
  while Found = 0 do
  begin
    if (SearchRec.Attr and faDirectory) <> 0 then begin
      // skip . and .. directories
      if SearchRec.Name [1] <> '.' then begin
        dtn := TDirTreeNode.Create;
        dtn.Load (Path + SearchRec.Name + '\');
        dtn.m_Name := SearchRec.Name;
        m_Space := m_Space + dtn.m_Space;
        m_Children.Add (dtn);
      end;

      // refresh progress bar in bigger steps (20K)
      if iCurrentSize - fmMain.pbScan.Position > 2*10240 then begin
        fmMain.pbScan.Position := iCurrentSize;
        fmMain.pbScan.Refresh;
      end
    end
    else if (SearchRec.Attr and faVolumeId)= 0 then begin
      m_Space := m_Space + SearchRec.Size;
      m_SpaceByFilesOnly := m_SpaceByFilesOnly + SearchRec.Size;
      iCurrentSize := iCurrentSize + SearchRec.Size;
    end;
    Found := FindNext(SearchRec);
  end;
  FindClose(SearchRec);
end;

procedure TDirTreeNode.Clear;
begin
  m_Space := 0;
  m_Children.Clear;
end;

procedure TDirTreeNode.ToTreeView (tnl: TTreeNodes; tn: TTreeNode; redAlert: boolean);
var
  newTn: TTreeNode;
  i, kidSpace: integer;
  alert: boolean;
  imIndex : integer;
begin
  newTn := tnl.AddChildObject (tn, m_Name+ ' : ' + IntToStr (m_Space div 1024) + ' K', Pointer (Self));

  if redAlert then
    imIndex := 1
  else
    imIndex := 0;

  newTn.SelectedIndex := imIndex;
  newTn.ImageIndex := imIndex;

  for i := 0 to m_Children.Count - 1 do begin
    // alert if used space by kid is > total space div number of kids (ave of kids)
    kidSpace := TDirTreeNode (m_Children.Items [i]).m_Space;
    alert := kidSpace > ( (m_Space-m_SpaceByFilesOnly) div m_Children.Count);
    TDirTreeNode (m_Children.Items [i]).ToTreeView (tnl, newTn, alert);
  end
end;

procedure TfmMain.FormCreate(Sender: TObject);
begin
  dirTree := TDirTreeNode.Create;
  flbFiles.Mask := '';
end;

procedure TfmMain.FormDestroy(Sender: TObject);
begin
  dirTree.Free;
end;

procedure TfmMain.bnRefreshClick(Sender: TObject);
var
  bDrive : byte;
  iDiskUsed : integer;
begin
  flbFiles.Drive := dcbDrives.Drive;
  flbFiles.Mask := '';

  bDrive := 1 + Ord (UpperCase (dcbDrives.Drive)[1]) - Ord ('A');
  iDiskUsed := DiskSize (bDrive) - DiskFree (bDrive);
  iCurrentSize := 0;

  pbScan.Position := 0;
  pbScan.Max := iDiskUsed;

  tvDir.Selected := nil;
  tvDir.Items.Clear;
  dirTree.Clear;

  dirTree.Load (dcbDrives.Drive + ':\');
  pbScan.Position := pbScan.Max;

  dirTree.ToTreeView (tvDir.Items, nil, false);
  tvDir.Items.Item [0].Expand (false);
end;

procedure TfmMain.tvDirChange(Sender: TObject; Node: TTreeNode);
var
  path: string;
begin
  if Node = nil then
    exit;
    
  path := '\';
  while Node.Parent <> nil do begin
    path := '\' + TDirTreeNode(Node.Data).m_Name + path;
    Node := Node.Parent;
  end;
  flbFiles.Directory := path;
  flbFiles.Mask := '*.*';
end;

procedure TfmMain.Button1Click(Sender: TObject);
begin
  AboutBox.ShowModal;
end;

procedure TfmMain.Button2Click(Sender: TObject);
begin
  fmHelp.ShowModal;
end;

end.
